;;;; unicode-utils.scm  -*- Hen -*-
;;;; Kon Lovett, Aug '10

;; Issues
;;
;; - Implies Unicode support that is not present.

(module unicode-utils

(;export
  ascii-codepoint?
  unicode-char->string
  unicode-string *unicode-string
  generic-make-string
  unicode-make-string *unicode-make-string
  unicode-surrogate?
  unicode-surrogates->codepoint)

(import scheme chicken)
(use
  (only srfi-1 every make-list)
  (only srfi-13 string-concatenate)
  (only type-checks check-natural-fixnum check-char))

(declare
  (bound-to-procedure
    ##sys#string-append
    ##sys#char->utf8-string
    ##sys#unicode-surrogate?
    ##sys#surrogates->codepoint))

;; Simple UTF 8

(: ascii-codepoint? (* -> boolean : char))
;
(define (ascii-codepoint? ch)
  (and
    (char? ch)
    (let ((x (char->integer ch)))
      (and (fx<= 0 x) (fx<= x #x7f)) ) ) )

(: unicode-char->string (char --> string))
;
(define (unicode-char->string ch)
  (##sys#char->utf8-string (check-char 'unicode-char->string ch)) )

;inefficient
(: unicode-string (#!rest (list-of char) --> string))
;
(define (unicode-string . chs)
  (if (null? chs)
    ""
    (begin
      (if (null? (cdr chs))
        (check-char 'unicode-string (car chs))
        (every (cut check-char 'unicode-string <>) chs) )
      (*unicode-string chs) ) ) )

(: generic-make-string (fixnum char --> string))
;
(define (generic-make-string len ch)
  (if (ascii-codepoint? ch)
    (make-string len ch)
    (unicode-make-string len ch) ) )

(: unicode-make-string (fixnum #!optional char --> string))
;
(define (unicode-make-string len #!optional (fill #\space))
  (*unicode-make-string
    (check-natural-fixnum 'unicode-make-string len)
    (check-char 'unicode-make-string fill)) )

(: unicode-surrogate? (* -> boolean : fixnum))
;
(define (unicode-surrogate? n)
  (and
    (fixnum? n)
    (##sys#unicode-surrogate? n) ) )

(: unicode-surrogates->codepoint (fixnum fixnum --> (or boolean fixnum)))
;
(define (unicode-surrogates->codepoint hi lo)
  (##sys#surrogates->codepoint
    (check-natural-fixnum 'unicode-surrogates->codepoint hi "high")
    (check-natural-fixnum 'unicode-surrogates->codepoint lo "low")) )

(: *unicode-make-string (fixnum char --> string))
;
(define (*unicode-make-string len fill)
  (cond
    ((fx= 0 len)
      "" )
    ((ascii-codepoint? fill)
      (##sys#make-string len fill) )
    (else
      (*unicode-string (make-list len fill)) ) ) )

;inefficient
(: *unicode-string ((list-of char) --> string))
;
(define (*unicode-string chs)
  (cond
    ((null? chs)
      "" )
    ((null? (cdr chs))
      (##sys#char->utf8-string (car chs)) )
    (else
      (string-concatenate (map ##sys#char->utf8-string chs)) ) ) )

) ;module unicode-utils
